@@ -1472,7 +1472,7 @@ ipmi_cmdlang_out_long(ipmi_cmd_info_t *info,
 void
 ipmi_cmdlang_out_binary(ipmi_cmd_info_t *info,
 			const char      *name,
-			char            *value,
+			const char      *value,
 			unsigned int    len)
 {
     info->cmdlang->out_binary(info->cmdlang, name, value, len);
@@ -1481,7 +1481,7 @@ ipmi_cmdlang_out_binary(ipmi_cmd_info_t *info,
 void
 ipmi_cmdlang_out_unicode(ipmi_cmd_info_t *info,
 			 const char      *name,
-			 char            *value,
+			 const char      *value,
 			 unsigned int    len)
 {
     info->cmdlang->out_unicode(info->cmdlang, name, value, len);
@@ -1491,7 +1491,7 @@ void
 ipmi_cmdlang_out_type(ipmi_cmd_info_t      *info,
 		      char                 *name,
 		      enum ipmi_str_type_e type,
-		      char                 *value,
+		      const char           *value,
 		      unsigned int         len)
 {
     switch(type) {
