@@ -98,10 +98,11 @@ redraw_cmdline(int force)
     }
 }
 
-void
-posix_vlog(char *format,
-	   enum ipmi_log_type_e log_type,
-	   va_list ap)
+static void
+my_vlog(os_handler_t         *handler,
+	const char           *format,
+	enum ipmi_log_type_e log_type,
+	va_list              ap)
 {
     int do_nl = 1;
     static int last_was_cont = 0;
@@ -153,13 +154,6 @@ posix_vlog(char *format,
 	redraw_cmdline(0);
     }
 }
-void
-debug_vlog(char *format,
-	   enum ipmi_log_type_e log_type,
-	   va_list ap)
-{
-    posix_vlog(format, log_type, ap);
-}
 
 #ifdef HAVE_GLIB
 void glib_handle_log(const gchar *log_domain,
@@ -1071,6 +1065,8 @@ main(int argc, char *argv[])
 #endif
     }
 
+    os_hnd->set_log_handler(os_hnd, my_vlog);
+
     /* Initialize the OpenIPMI library. */
     ipmi_init(os_hnd);
 
