@@ -36,6 +36,10 @@
 
 #include <OpenIPMI/ipmi_types.h>
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 /* The abstract type for lanparm. */
 typedef struct ipmi_lanparm_s ipmi_lanparm_t;
 
@@ -45,6 +49,10 @@ typedef void (*ipmi_lanparm_done_cb)(ipmi_lanparm_t *lanparm,
 				     int            err,
 				     void           *cb_data);
 
+/* Generic callback for iterating. */
+typedef void (*ipmi_lanparm_ptr_cb)(ipmi_lanparm_t *lanparm,
+				    void           *cb_data);
+
 /* Allocate a LANPARM. */
 int ipmi_lanparm_alloc(ipmi_mc_t      *mc,
 		       unsigned int   channel,
@@ -55,6 +63,26 @@ int ipmi_lanparm_destroy(ipmi_lanparm_t       *lanparm,
 			 ipmi_lanparm_done_cb handler,
 			 void                 *cb_data);
 
+/* Used to track references to a lanparm.  You can use this instead of
+   ipmi_lanparm_destroy, but use of the destroy function is
+   recommended.  This is primarily here to help reference-tracking
+   garbage collection systems like what is in Perl to be able to
+   automatically destroy lanparms when they are done. */
+void ipmi_lanparm_ref(ipmi_lanparm_t *lanparm);
+void ipmi_lanparm_deref(ipmi_lanparm_t *lanparm);
+
+void ipmi_lanparm_iterate_lanparms(ipmi_domain_t       *domain,
+				   ipmi_lanparm_ptr_cb handler,
+				   void                *cb_data);
+
+ipmi_mcid_t ipmi_lanparm_get_mc_id(ipmi_lanparm_t *lanparm);
+unsigned int ipmi_lanparm_get_channel(ipmi_lanparm_t *lanparm);
+
+#define IPMI_LANPARM_NAME_LEN 64
+int ipmi_lanparm_get_name(ipmi_lanparm_t *lanparm, char *name, int length);
+
+
+
 /* Fetch a parameter value from the LANPARM.  The "set" and "block"
    parameters are the set selector and block selectors.  If those are
    not relevant for the given parm, then set them to zero.  Note that
@@ -101,6 +129,12 @@ int ipmi_lanparm_set_parm(ipmi_lanparm_t       *lanparm,
 #define IPMI_LANPARM_NUM_DESTINATIONS		17
 #define IPMI_LANPARM_DEST_TYPE			18
 #define IPMI_LANPARM_DEST_ADDR			19
+#define IPMI_LANPARM_VLAN_ID			20
+#define IPMI_LANPARM_VLAN_PRIORITY		21
+#define IPMI_LANPARM_NUM_CIPHER_SUITE_ENTRIES	22
+#define IPMI_LANPARM_CIPHER_SUITE_ENTRY_SUPPORT	23
+#define IPMI_LANPARM_CIPHER_SUITE_ENTRY_PRIV	24
+#define IPMI_LANPARM_DEST_VLAN_TAG		25
 
 /* A full LAN configuration.  Note that you cannot allocate one of
    these, you can only fetch them, modify them, set them, and free
@@ -145,6 +179,59 @@ void ipmi_lan_free_config(ipmi_lan_config_t *config);
  * ports, etc. are in network order.
  */
 
+/* This interface lets you fetch and set the data values by parm
+   num. Note that the parm nums *DO NOT* correspond to the
+   IPMI_LANPARM_xxx values above. */
+
+enum ipmi_lanconf_val_type_e { IPMI_LANCONFIG_INT, IPMI_LANCONFIG_BOOL,
+			       IPMI_LANCONFIG_DATA,
+			       IPMI_LANCONFIG_IP, IPMI_LANCONFIG_MAC };
+/* When getting the value, the valtype will be set to int or data.  If
+   it is int or bool, the value is returned in ival and the dval is
+   not used.  If it is data, ip, or mac, the data will be returned in
+   an allocated array in dval and the length set in dval_len.  The
+   data must be freed with ipmi_lanconfig_data_free().  The is used
+   for some data items (the priv level for authentication type, the
+   destination for alerts and destination addresses); for other items
+   it is ignored.  The index should point to the value to fetch
+   (starting at zero), it will be updated to the next value or -1 if
+   no more are left.  The index will be unchanged if the parm does not
+   support an index.
+
+   The string name is returned in the name field, if it is not NULL.
+
+   Note that when fetching a value, if the passed in pointer is NULL
+   the data will not be filled in (except for index, which must always
+   be present).  That lets you get the value type without getting the
+   data, for instance. */
+int ipmi_lanconfig_get_val(ipmi_lan_config_t *lanc,
+			   unsigned int      parm,
+			   const char        **name,
+			   int               *index,
+			   enum ipmi_lanconf_val_type_e *valtype,
+			   unsigned int      *ival,
+			   unsigned char     **dval,
+			   unsigned int      *dval_len);
+  /* Set a value in the lan config.  You must know ahead of time the
+     actual value type and set the proper one. */
+int ipmi_lanconfig_set_val(ipmi_lan_config_t *lanc,
+			   unsigned int      parm,
+			   int               index,
+			   unsigned int      ival,
+			   unsigned char     *dval,
+			   unsigned int      dval_len);
+/* Free data from ipmi_lanconfig_get_val(). */
+void ipmi_lanconfig_data_free(void *data);
+/* Convert a string to a lanconfig parm number.  Returns -1 if the
+   string is invalid. */
+unsigned int ipmi_lanconfig_str_to_parm(char *name);
+/* Convert the parm to a string name. */
+const char *ipmi_lanconfig_parm_to_str(unsigned int parm);
+/* Get the type of a specific parm. */
+int ipmi_lanconfig_parm_to_type(unsigned int                 parm,
+				enum ipmi_lanconf_val_type_e *valtype);
+
+
 /* The first set of parameters here must be present, so they are
    returned directly, no errors for getting.  Setting returns an
    error. */
@@ -161,7 +248,10 @@ unsigned int ipmi_lanconfig_get_ip_addr_source(ipmi_lan_config_t *lanc);
 int ipmi_lanconfig_set_ip_addr_source(ipmi_lan_config_t *lanc,
 				      unsigned int      val);
 
-/* Number of allowed alert destinations.  This is read-only. */
+/* Number of allowed alert destinations.  This is read-only.  Note
+   that this is *not* the value returned from the config, it is one
+   more than the value, so it is the actual number of alert
+   destinations (including the volatile destination 0). */
 unsigned int ipmi_lanconfig_get_num_alert_destinations(ipmi_lan_config_t *c);
 
 
@@ -185,34 +275,34 @@ int ipmi_lanconfig_set_ipv4_tos(ipmi_lan_config_t *lanc,
 
 /* Authorization enables for the various authentication levels. */
 int ipmi_lanconfig_get_enable_auth_oem(ipmi_lan_config_t *lanc,
-				       unsigned int      auth,
+				       unsigned int      user,
 				       unsigned int      *val);
 int ipmi_lanconfig_get_enable_auth_straight(ipmi_lan_config_t *lanc,
-					    unsigned int      auth,
+					    unsigned int      user,
 					    unsigned int      *val);
 int ipmi_lanconfig_get_enable_auth_md5(ipmi_lan_config_t *lanc,
-				       unsigned int      auth,
+				       unsigned int      user,
 				       unsigned int      *val);
 int ipmi_lanconfig_get_enable_auth_md2(ipmi_lan_config_t *lanc,
-				       unsigned int      auth,
+				       unsigned int      user,
 				       unsigned int      *val);
 int ipmi_lanconfig_get_enable_auth_none(ipmi_lan_config_t *lanc,
-					unsigned int      auth,
+					unsigned int      user,
 					unsigned int      *val);
 int ipmi_lanconfig_set_enable_auth_oem(ipmi_lan_config_t *lanc,
-				       unsigned int      auth,
+				       unsigned int      user,
 				       unsigned int      val);
 int ipmi_lanconfig_set_enable_auth_straight(ipmi_lan_config_t *lanc,
-					    unsigned int      auth,
+					    unsigned int      user,
 					    unsigned int      val);
 int ipmi_lanconfig_set_enable_auth_md5(ipmi_lan_config_t *lanc,
-				       unsigned int      auth,
+				       unsigned int      user,
 				       unsigned int      val);
 int ipmi_lanconfig_set_enable_auth_md2(ipmi_lan_config_t *lanc,
-				       unsigned int      auth,
+				       unsigned int      user,
 				       unsigned int      val);
 int ipmi_lanconfig_set_enable_auth_none(ipmi_lan_config_t *lanc,
-					unsigned int      auth,
+					unsigned int      user,
 					unsigned int      val);
 
 /* Addressing for the BMC. */
@@ -243,12 +333,20 @@ int ipmi_lanconfig_get_primary_rmcp_port(ipmi_lan_config_t *lanc,
 int ipmi_lanconfig_set_primary_rmcp_port(ipmi_lan_config_t *lanc,
 					 unsigned char     *data,
 					 unsigned int      data_len);
+int ipmi_lanconfig_get_port_rmcp_primary(ipmi_lan_config_t *lanc,
+					 unsigned int      *val);
+int ipmi_lanconfig_set_port_rmcp_primary(ipmi_lan_config_t *lanc,
+					 unsigned int      val);
 int ipmi_lanconfig_get_secondary_rmcp_port(ipmi_lan_config_t *lanc,
 					   unsigned char     *data,
 					   unsigned int      *data_len);
 int ipmi_lanconfig_set_secondary_rmcp_port(ipmi_lan_config_t *lanc,
 					   unsigned char     *data,
 					   unsigned int      data_len);
+int ipmi_lanconfig_get_port_rmcp_secondary(ipmi_lan_config_t *lanc,
+					   unsigned int      *val);
+int ipmi_lanconfig_set_port_rmcp_secondary(ipmi_lan_config_t *lanc,
+					   unsigned int      val);
 
 /* Control of ARP-ing.  These are optional and so may return errors. */
 int ipmi_lanconfig_get_bmc_generated_arps(ipmi_lan_config_t *lanc,
@@ -352,5 +450,51 @@ int ipmi_lanconfig_set_dest_mac_addr(ipmi_lan_config_t *lanc,
 				     unsigned int      dest,
 				     unsigned char     *data,
 				     unsigned int      data_len);
+int ipmi_lanconfig_get_dest_vlan_tag_type(ipmi_lan_config_t *lanc,
+					  unsigned int      dest,
+					  unsigned int      *val);
+int ipmi_lanconfig_set_dest_vlan_tag_type(ipmi_lan_config_t *lanc,
+					  unsigned int      dest,
+					  unsigned int      val);
+int ipmi_lanconfig_get_dest_vlan_tag(ipmi_lan_config_t *lanc,
+				     unsigned int      dest,
+				     unsigned int      *val);
+int ipmi_lanconfig_set_dest_vlan_tag(ipmi_lan_config_t *lanc,
+				     unsigned int      dest,
+				     unsigned int      val);
+
+/* VLAN support */
+int ipmi_lanconfig_get_vlan_id(ipmi_lan_config_t *lanc,
+			       unsigned int      *val);
+int ipmi_lanconfig_set_vlan_id(ipmi_lan_config_t *lanc,
+			       unsigned int      val);
+int ipmi_lanconfig_get_vlan_id_enable(ipmi_lan_config_t *lanc,
+				      unsigned int      *val);
+int ipmi_lanconfig_set_vlan_id_enable(ipmi_lan_config_t *lanc,
+				      unsigned int      val);
+int ipmi_lanconfig_get_vlan_priority(ipmi_lan_config_t *lanc,
+				     unsigned int      *val);
+int ipmi_lanconfig_set_vlan_priority(ipmi_lan_config_t *lanc,
+				     unsigned int      val);
+
+/* Cipher Suites */
+unsigned int ipmi_lanconfig_get_num_cipher_suites(ipmi_lan_config_t *lanc);
+					   
+int ipmi_lanconfig_get_cipher_suite_entry(ipmi_lan_config_t *lanc,
+					  unsigned int      entry,
+					  unsigned int      *val);
+int ipmi_lanconfig_set_cipher_suite_entry(ipmi_lan_config_t *lanc,
+					  unsigned int      entry,
+					  unsigned int      val);
+int ipmi_lanconfig_get_max_priv_for_cipher_suite(ipmi_lan_config_t *lanc,
+						 unsigned int      entry,
+						 unsigned int      *val);
+int ipmi_lanconfig_set_max_priv_for_cipher_suite(ipmi_lan_config_t *lanc,
+						 unsigned int      entry,
+						 unsigned int      val);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif /* _IPMI_LANPARM_H */
