@@ -170,6 +170,14 @@ typedef struct led_data_s
     unsigned char def_override_color;
 } led_data_t;
 
+#define IPMI_MAX_CHANNELS 8
+typedef struct lmc_channel_info_s
+{
+    unsigned char medium_type;
+    unsigned char protocol_type;
+    unsigned char session_support;
+} lmc_channel_info_t;
+
 struct lmc_data_s
 {
     emu_data_t *emu;
@@ -188,6 +196,8 @@ struct lmc_data_s
     unsigned char mfg_id[3];	   /* bytes 8-10 */
     unsigned char product_id[2];   /* bytes 11-12 */
 
+    lmc_channel_info_t chans[IPMI_MAX_CHANNELS];
+
     sel_t sel;
 
     sdrs_t main_sdrs;
@@ -299,7 +309,8 @@ ipmi_mc_enable_sel(lmc_data_t    *mc,
 int
 ipmi_mc_add_to_sel(lmc_data_t    *mc,
 		   unsigned char record_type,
-		   unsigned char event[13])
+		   unsigned char event[13],
+		   unsigned int  *recid)
 {
     sel_entry_t    *e;
     struct timeval t;
@@ -352,6 +363,9 @@ ipmi_mc_add_to_sel(lmc_data_t    *mc,
     mc->sel.count++;
 
     mc->sel.last_add_time = t.tv_sec + mc->sel.time_offset;
+
+    if (recid)
+      *recid = e->record_id;
     return 0;
 }
 
@@ -535,7 +549,8 @@ handle_add_sel_entry(lmc_data_t    *mc,
 		     unsigned char *rdata,
 		     unsigned int  *rdata_len)
 {
-    int rv;
+    int          rv;
+    unsigned int r;
 
     if (!(mc->device_support & IPMI_DEVID_SEL_DEVICE)) {
 	handle_invalid_cmd(mc, rdata, rdata_len);
@@ -545,19 +560,16 @@ handle_add_sel_entry(lmc_data_t    *mc,
     if (check_msg_length(msg, 16, rdata, rdata_len))
 	return;
 
-    if (msg->data[2] < 0xe0)
-	rv = ipmi_mc_add_to_sel(mc, msg->data[2], msg->data+6);
-    else
-	rv = ipmi_mc_add_to_sel(mc, msg->data[2], msg->data+3);
-
+    rv = ipmi_mc_add_to_sel(mc, msg->data[2], msg->data+3, &r);
     if (rv == EAGAIN) {
 	rdata[0] = IPMI_OUT_OF_SPACE_CC;
     } else if (rv) {
 	rdata[0] = IPMI_UNKNOWN_ERR_CC;
     } else {
 	rdata[0] = 0;
+	ipmi_set_uint16(rdata+1, r);
     }
-    *rdata_len = 1;
+    *rdata_len = 3;
 }
 
 static void
@@ -1532,6 +1544,26 @@ handle_write_fru_data(lmc_data_t    *mc,
     *rdata_len = 2;
 }
 
+int
+ipmi_mc_get_fru_data(lmc_data_t    *mc,
+		     unsigned char device_id,
+		     unsigned int  *length,
+		     unsigned char **data)
+{
+    if (!(mc->device_support & IPMI_DEVID_FRU_INVENTORY_DEV))
+	return ENOSYS;
+
+    if (device_id >= 255)
+	return EINVAL;
+
+    if (!mc->frus[device_id].data)
+	return EINVAL;
+
+    *data = mc->frus[device_id].data;
+    *length = mc->frus[device_id].length;
+    return 0;
+}
+
 int
 ipmi_mc_add_fru_data(lmc_data_t    *mc,
 		     unsigned char device_id,
@@ -1694,21 +1726,70 @@ handle_get_device_id(lmc_data_t    *mc,
     *rdata_len = 12;
 }
 
+static void
+handle_get_channel_info(lmc_data_t    *mc,
+			unsigned char chan,
+			ipmi_msg_t    *msg,
+			unsigned char *rdata,
+			unsigned int  *rdata_len)
+{
+    unsigned char lchan;
+
+    if (msg->data_len < 1) {
+	rdata[0] = IPMI_REQUEST_DATA_LENGTH_INVALID_CC;
+	*rdata_len = 1;
+	return;
+    }
+
+    lchan = msg->data[0];
+    if (lchan == 0xe)
+	lchan = chan;
+    else if (lchan >= IPMI_MAX_CHANNELS) {
+	rdata[0] = IPMI_INVALID_DATA_FIELD_CC;
+	*rdata_len = 1;
+	return;
+    }
+
+    if (! mc->chans[lchan].medium_type) {
+	rdata[0] = IPMI_NOT_PRESENT_CC;
+	*rdata_len = 1;
+	return;
+    }
+
+    rdata[0] = 0;
+    rdata[1] = lchan;
+    rdata[2] = mc->chans[lchan].medium_type;
+    rdata[3] = mc->chans[lchan].protocol_type;
+    /* FIXME - no handling of active sessions */
+    rdata[4] = mc->chans[lchan].session_support << 6;
+    rdata[5] = 0xf2;
+    rdata[6] = 0x1b;
+    rdata[7] = 0x00;
+    rdata[8] = 0x00;
+    rdata[9] = 0x00;
+    *rdata_len = 10;
+}
+
 static void
 handle_app_netfn(lmc_data_t    *mc,
+		 unsigned char chan,
 		 unsigned char lun,
 		 ipmi_msg_t    *msg,
 		 unsigned char *rdata,
 		 unsigned int  *rdata_len)
 {
     switch(msg->cmd) {
-	case IPMI_GET_DEVICE_ID_CMD:
-	    handle_get_device_id(mc, msg, rdata, rdata_len);
-	    break;
+    case IPMI_GET_DEVICE_ID_CMD:
+	handle_get_device_id(mc, msg, rdata, rdata_len);
+	break;
 
-	default:
-	    handle_invalid_cmd(mc, rdata, rdata_len);
-	    break;
+    case IPMI_GET_CHANNEL_INFO_CMD:
+	handle_get_channel_info(mc, chan, msg, rdata, rdata_len);
+	break;
+
+    default:
+	handle_invalid_cmd(mc, rdata, rdata_len);
+	break;
     }
 }
 
@@ -1993,7 +2074,7 @@ do_event(lmc_data_t    *mc,
     data[11] = byte2;
     data[12] = byte3;
 
-    ipmi_mc_add_to_sel(dest_mc, 0x02, data);
+    ipmi_mc_add_to_sel(dest_mc, 0x02, data, NULL);
 }
 
 static void
@@ -2234,6 +2315,8 @@ handle_set_sensor_event_enable(lmc_data_t    *mc,
 	if (msg->data_len <= i)
 	    break;
 	for (j=0; j<8; j++, e++) {
+	    if (e >= 15)
+		break;
 	    if ((msg->data[i] >> j) & 1)
 		sensor->event_enabled[0][e] = op;
 	}
@@ -2243,6 +2326,8 @@ handle_set_sensor_event_enable(lmc_data_t    *mc,
 	if (msg->data_len <= i)
 	    break;
 	for (j=0; j<8; j++, e++) {
+	    if (e >= 15)
+		break;
 	    if ((msg->data[i] >> j) & 1)
 		sensor->event_enabled[1][e] = op;
 	}
@@ -2294,14 +2379,20 @@ handle_get_sensor_event_enable(lmc_data_t    *mc,
     e = 0;
     for (i=2; i<=3; i++) {
 	rdata[i] = 0;
-	for (j=0; j<8; j++, e++)
+	for (j=0; j<8; j++, e++) {
+	    if (e >= 15)
+		break;
 	    rdata[i] |= sensor->event_enabled[0][e] << j;
+	}
     }
     e = 0;
     for (i=4; i<=5; i++) {
 	rdata[i] = 0;
-	for (j=0; j<8; j++, e++)
+	for (j=0; j<8; j++, e++) {
+	    if (e >= 15)
+		break;
 	    rdata[i] |= sensor->event_enabled[1][e] << j;
+	}
     }
 
     *rdata_len = 6;
@@ -2374,8 +2465,11 @@ handle_get_sensor_reading(lmc_data_t    *mc,
     e = 0;
     for (i=3; i<=4; i++) {
 	rdata[i] = 0;
-	for (j=0; j<8; j++, e++)
+	for (j=0; j<8; j++, e++) {
+	    if (e >= 15)
+		break;
 	    rdata[i] |= sensor->event_status[e] << j;
+	}
     }
     *rdata_len = 5;
 }
@@ -2678,7 +2772,7 @@ ipmi_mc_set_power(lmc_data_t *mc, unsigned char power, int gen_event)
     data[11] = 0;
     data[12] = 0;
 
-    ipmi_mc_add_to_sel(dest_mc, 0xc0, data);
+    ipmi_mc_add_to_sel(dest_mc, 0xc0, data, NULL);
 	
     return 0;
 }
@@ -3096,7 +3190,7 @@ handle_picmg_cmd_get_fru_led_state(lmc_data_t    *mc,
     rdata[1] = IPMI_PICMG_GRP_EXT;
     rdata[2] = 0x00;
     if (mc->leds[led].loc_cnt_sup)
-	rdata[2] |= 0x04; /* Local control support */
+	rdata[2] |= 0x01; /* Local control support */
 
     if (mc->leds[led].loc_cnt) {
 	rdata[3] = mc->leds[led].off_dur;
@@ -3493,6 +3587,7 @@ ipmb_checksum(uint8_t *data, int size, uint8_t start)
 
 void
 ipmi_emu_handle_msg(emu_data_t     *emu,
+		    unsigned char  chan,
 		    unsigned char  lun,
 		    ipmi_msg_t     *msg,
 		    unsigned char  *rdata,
@@ -3543,6 +3638,7 @@ ipmi_emu_handle_msg(emu_data_t     *emu,
 	smsg.data_len = data_len - 7; /* Subtract off the header and
 					 the end checksum */
 	msg = &smsg;
+	chan = 0; /* IPMB channel is 0 */
     } else {
 	mc = emu->ipmb[emu->bmc_mc >> 1];
 	if (!mc || !mc->enabled) {
@@ -3554,7 +3650,7 @@ ipmi_emu_handle_msg(emu_data_t     *emu,
 
     switch (msg->netfn) {
     case IPMI_APP_NETFN:
-	handle_app_netfn(mc, lun, msg, rdata, rdata_len);
+	handle_app_netfn(mc, chan, lun, msg, rdata, rdata_len);
 	break;
 
     case IPMI_SENSOR_EVENT_NETFN:
@@ -3635,6 +3731,21 @@ ipmi_mc_destroy(lmc_data_t *mc)
     free(mc);
 }
 
+int
+ipmi_emu_set_mc_channel(lmc_data_t    *mc,
+			unsigned char channel,
+			unsigned char medium_type,
+			unsigned char protocol_type,
+			unsigned char session_support)
+{
+    if (channel >= IPMI_MAX_CHANNELS)
+	return EINVAL;
+    mc->chans[channel].medium_type = medium_type;
+    mc->chans[channel].protocol_type = protocol_type;
+    mc->chans[channel].session_support = session_support & 0x3;
+    return 0;
+}
+
 void
 ipmi_mc_disable(lmc_data_t *mc)
 {
