@@ -35,14 +35,15 @@
 #include <string.h>
 
 #include <OpenIPMI/ipmiif.h>
-#include <OpenIPMI/ipmi_entity.h>
-#include <OpenIPMI/ipmi_control.h>
-#include <OpenIPMI/ipmi_mc.h>
 #include <OpenIPMI/ipmi_err.h>
-#include <OpenIPMI/ipmi_int.h>
 #include <OpenIPMI/ipmi_msgbits.h>
-#include <OpenIPMI/ipmi_domain.h>
-#include <OpenIPMI/ipmi_oem.h>
+
+#include <OpenIPMI/internal/ipmi_domain.h>
+#include <OpenIPMI/internal/ipmi_oem.h>
+#include <OpenIPMI/internal/ipmi_int.h>
+#include <OpenIPMI/internal/ipmi_entity.h>
+#include <OpenIPMI/internal/ipmi_control.h>
+#include <OpenIPMI/internal/ipmi_mc.h>
 
 static int
 chassis_entity_sdr_add(ipmi_entity_t   *ent,
@@ -352,13 +353,22 @@ chassis_power_get(ipmi_control_t      *control,
 }
 
 static void
-chassis_mc_control_removal_handler(ipmi_domain_t *domain,
-				   ipmi_mc_t     *mc,
-				   void          *cb_data)
+chassis_mc_control_active_handler(ipmi_mc_t *mc,
+				  int       active,
+				  void      *cb_data)
 {
     ipmi_control_t *control = cb_data;
+    ipmi_entity_t  *entity = ipmi_control_get_entity(control);
+    ipmi_domain_t  *domain = ipmi_mc_get_domain(mc);
 
+    if (active)
+	return;
+
+    _ipmi_domain_entity_lock(domain);
+    _ipmi_entity_get(entity);
+    _ipmi_domain_entity_unlock(domain);
     ipmi_control_destroy(control);
+    _ipmi_entity_put(entity);
 }
 
 int
@@ -366,9 +376,9 @@ _ipmi_chassis_create_controls(ipmi_mc_t *mc)
 {
     ipmi_domain_t      *domain = ipmi_mc_get_domain(mc);
     ipmi_entity_info_t *ents = ipmi_domain_get_entities(domain);
-    ipmi_entity_t      *chassis_ent;
-    ipmi_control_t     *power_control;
-    ipmi_control_t     *reset_control;
+    ipmi_entity_t      *chassis_ent = NULL;
+    ipmi_control_t     *power_control = NULL;
+    ipmi_control_t     *reset_control = NULL;
     int                rv;
     ipmi_control_cbs_t cbs;
 
@@ -384,7 +394,7 @@ _ipmi_chassis_create_controls(ipmi_mc_t *mc)
 		 DOMAIN_NAME(domain), rv);
 	goto out;
     }
-
+    
     /* Allocate the power control. */
     rv = ipmi_control_alloc_nonstandard(&power_control);
     if (rv) {
@@ -415,9 +425,9 @@ _ipmi_chassis_create_controls(ipmi_mc_t *mc)
 	goto out;
     }
 
-    rv = ipmi_mc_add_oem_removed_handler(mc,
-					 chassis_mc_control_removal_handler,
-					 power_control, NULL);
+    rv = ipmi_mc_add_active_handler(mc,
+				    chassis_mc_control_active_handler,
+				    power_control);
     if (rv) {
 	ipmi_control_destroy(power_control);
 	goto out;
@@ -452,14 +462,20 @@ _ipmi_chassis_create_controls(ipmi_mc_t *mc)
 	goto out;
     }
 
-    rv = ipmi_mc_add_oem_removed_handler(mc,
-					 chassis_mc_control_removal_handler,
-					 reset_control, NULL);
+    rv = ipmi_mc_add_active_handler(mc,
+				    chassis_mc_control_active_handler,
+				    reset_control);
     if (rv) {
 	ipmi_control_destroy(reset_control);
 	goto out;
     }
 
  out:
+    if (power_control)
+	_ipmi_control_put(power_control);
+    if (reset_control)
+	_ipmi_control_put(reset_control);
+    if (chassis_ent)
+	_ipmi_entity_put(chassis_ent);
     return rv;
 }
