@@ -274,12 +274,12 @@ smi_put(ipmi_con_t *ipmi)
 
 /* Must be called with cmd_lock held. */
 static void
-add_cmd(ipmi_con_t    *ipmi,
-	ipmi_addr_t   *addr,
-	unsigned int  addr_len,
-	ipmi_msg_t    *msg,
-	smi_data_t    *smi,
-	pending_cmd_t *cmd)
+add_cmd(ipmi_con_t        *ipmi,
+	const ipmi_addr_t *addr,
+	unsigned int      addr_len,
+	const ipmi_msg_t  *msg,
+	smi_data_t        *smi,
+	pending_cmd_t     *cmd)
 {
     cmd->ipmi = ipmi;
     memcpy(&(cmd->addr), addr, addr_len);
@@ -420,12 +420,12 @@ open_smi_fd(int if_num, int *using_socket)
 }
 
 static int
-smi_send(smi_data_t   *smi,
-	 int          fd,
-	 ipmi_addr_t  *addr,
-	 unsigned int addr_len,
-	 ipmi_msg_t   *msg,
-	 long         msgid)
+smi_send(smi_data_t        *smi,
+	 int               fd,
+	 const ipmi_addr_t *addr,
+	 unsigned int      addr_len,
+	 const ipmi_msg_t  *msg,
+	 long              msgid)
 {
     int rv;
     ipmi_addr_t myaddr;
@@ -711,10 +711,10 @@ handle_response(ipmi_con_t *ipmi, struct ipmi_recv *recv)
 
 typedef struct call_event_handler_s
 {
-    smi_data_t   *smi;
-    ipmi_addr_t  *addr;
-    unsigned int addr_len;
-    ipmi_event_t *event;
+    smi_data_t        *smi;
+    const ipmi_addr_t *addr;
+    unsigned int      addr_len;
+    ipmi_event_t      *event;
 } call_event_handler_t;
 
 static int
@@ -780,10 +780,10 @@ smi_remove_event_handler(ipmi_con_t            *ipmi,
 static ipmi_mcid_t invalid_mcid = IPMI_MCID_INVALID;
 
 static void
-handle_async_event(ipmi_con_t  *ipmi,
-		   ipmi_addr_t *addr,
-		   unsigned int addr_len,
-		   ipmi_msg_t   *msg)
+handle_async_event(ipmi_con_t        *ipmi,
+		   const ipmi_addr_t *addr,
+		   unsigned int      addr_len,
+		   const ipmi_msg_t  *msg)
 {
     smi_data_t           *smi = (smi_data_t *) ipmi->con_data;
     ipmi_event_t         *event;
@@ -1012,9 +1012,9 @@ ipmi_sock_data_handler(int            fd,
 
 static int
 smi_send_command(ipmi_con_t            *ipmi,
-		 ipmi_addr_t           *addr,
+		 const ipmi_addr_t     *addr,
 		 unsigned int          addr_len,
-		 ipmi_msg_t            *msg,
+		 const ipmi_msg_t      *msg,
 		 ipmi_ll_rsp_handler_t rsp_handler,
 		 ipmi_msgi_t           *trspi)
 {
@@ -1098,11 +1098,11 @@ smi_send_command(ipmi_con_t            *ipmi,
 }
 
 static int
-smi_send_response(ipmi_con_t   *ipmi,
-		  ipmi_addr_t  *addr,
-		  unsigned int addr_len,
-		  ipmi_msg_t   *msg,
-		  long         sequence)
+smi_send_response(ipmi_con_t        *ipmi,
+		  const ipmi_addr_t *addr,
+		  unsigned int      addr_len,
+		  const ipmi_msg_t  *msg,
+		  long              sequence)
 {
     smi_data_t *smi;
     int        rv;
