@@ -36,13 +36,15 @@
 #include <errno.h>
 
 #include <OpenIPMI/ipmiif.h>
-#include <OpenIPMI/ipmi_oem.h>
-#include <OpenIPMI/ipmi_sensor.h>
-#include <OpenIPMI/ipmi_control.h>
-#include <OpenIPMI/ipmi_int.h>
 #include <OpenIPMI/ipmi_err.h>
-#include <OpenIPMI/ipmi_domain.h>
-#include <OpenIPMI/ipmi_event.h>
+
+#include <OpenIPMI/internal/ipmi_event.h>
+#include <OpenIPMI/internal/ipmi_domain.h>
+#include <OpenIPMI/internal/ipmi_mc.h>
+#include <OpenIPMI/internal/ipmi_oem.h>
+#include <OpenIPMI/internal/ipmi_sensor.h>
+#include <OpenIPMI/internal/ipmi_control.h>
+#include <OpenIPMI/internal/ipmi_int.h>
 
 /* We use a block of MontaVista's private enterprise IDs for our own
    purposes. */
@@ -174,6 +176,8 @@ test_event_handler_0(ipmi_mc_t    *mc,
 	if (!rv)
 	    rv = info.err;
 
+	_ipmi_mc_put(src_mc);
+
 	if (!rv)
 	    return 1;
     }
@@ -630,14 +634,14 @@ static ipmi_control_transition_t blue_led2[] =
 };
 
 
-static ipmi_control_setting_t hs_led_settings[] =
+static ipmi_control_value_t hs_led_values[] =
 {
     { 2, blue_led1 },
     { 1, off_led },
     { 2, blue_led2 },
     { 1, on_blue_led },
 };
-static ipmi_control_light_t hs_led[] = {{ 4, hs_led_settings }};
+static ipmi_control_light_t hs_led[] = {{ 4, hs_led_values }};
 
 static int
 test_handler_0(ipmi_mc_t *mc,
@@ -645,7 +649,7 @@ test_handler_0(ipmi_mc_t *mc,
 {
     ipmi_domain_t      *domain = ipmi_mc_get_domain(mc);
     ipmi_entity_info_t *ents = ipmi_domain_get_entities(domain);
-    ipmi_entity_t      *ent;
+    ipmi_entity_t      *ent = NULL;
     ipmi_control_t     *control;
     int                rv = 0;
     ipmi_control_cbs_t cbs;
@@ -693,7 +697,7 @@ test_handler_0(ipmi_mc_t *mc,
     }
 
     ipmi_control_set_type(control, IPMI_CONTROL_POWER);
-    ipmi_control_set_ignore_if_no_entity(control, 0);
+    ipmi_control_set_ignore_if_no_entity(control, 1);
     ipmi_control_set_id(control, "power", IPMI_ASCII_STR, 5);
     ipmi_control_set_hot_swap_power(control, 1);
 
@@ -724,7 +728,7 @@ test_handler_0(ipmi_mc_t *mc,
 
     rv = ipmi_mc_add_oem_removed_handler(mc,
 					 mc_control_removal_handler,
-					 control, NULL);
+					 control);
     if (rv) {
 	ipmi_log(IPMI_LOG_WARNING,
 		 "%soem_test.c(test_handler_0): "
@@ -741,7 +745,7 @@ test_handler_0(ipmi_mc_t *mc,
     }
 
     ipmi_control_set_type(control, IPMI_CONTROL_LIGHT);
-    ipmi_control_set_ignore_if_no_entity(control, 0);
+    ipmi_control_set_ignore_if_no_entity(control, 1);
     ipmi_control_set_id(control, "Hotswap LED", IPMI_ASCII_STR, 11);
     ipmi_control_light_set_lights(control, 1, hs_led);
     ipmi_control_set_hot_swap_indicator(control, 1, 0, 1, 2, 3);
@@ -768,22 +772,28 @@ test_handler_0(ipmi_mc_t *mc,
 		 "Could not add the power control: %x",
 		 MC_NAME(mc), rv);
 	ipmi_control_destroy(control);
+	_ipmi_control_put(control);
 	goto out;
     }
 
     rv = ipmi_mc_add_oem_removed_handler(mc,
 					 mc_control_removal_handler,
-					 control, NULL);
+					 control);
     if (rv) {
 	ipmi_log(IPMI_LOG_WARNING,
 		 "%soem_test.c(test_handler_0): "
 		 "Could not add the power control removal handler: %x",
 		 MC_NAME(mc), rv);
 	ipmi_control_destroy(control);
+	_ipmi_control_put(control);
 	goto out;
     }
 
+    _ipmi_control_put(control);
+
  out:
+    if (ent)
+	_ipmi_entity_put(ent);
     return rv;
 }
 
