@@ -41,20 +41,29 @@
 
 #include <string.h>
 #include <math.h>
+#include <stdio.h>
 
 #include <OpenIPMI/ipmiif.h>
 #include <OpenIPMI/ipmi_pef.h>
 #include <OpenIPMI/ipmi_msgbits.h>
-#include <OpenIPMI/ipmi_domain.h>
-#include <OpenIPMI/ipmi_mc.h>
 #include <OpenIPMI/ipmi_err.h>
-#include <OpenIPMI/ipmi_int.h>
-#include <OpenIPMI/ilist.h>
-#include <OpenIPMI/opq.h>
+
+#include <OpenIPMI/internal/opq.h>
+#include <OpenIPMI/internal/locked_list.h>
+#include <OpenIPMI/internal/ipmi_int.h>
+#include <OpenIPMI/internal/ipmi_domain.h>
+#include <OpenIPMI/internal/ipmi_mc.h>
+
+#define IPMI_PEF_ATTR_NAME "ipmi_pef"
 
 struct ipmi_pef_s
 {
     ipmi_mcid_t mc;
+    ipmi_domain_id_t domain;
+
+    int refcount;
+
+    char name[IPMI_PEF_NAME_LEN];
 
     /* Is the PEF ready (the capabilities have been checked)? */
     unsigned int ready : 1;
@@ -62,6 +71,8 @@ struct ipmi_pef_s
     /* Does the MC have a usable PEF? */
     unsigned int valid : 1;
 
+    unsigned int in_list : 1;
+
     /* Information from the get PEF capability command. */
     unsigned int can_diagnostic_interrupt : 1;
     unsigned int can_oem_action : 1;
@@ -94,6 +105,19 @@ struct ipmi_pef_s
     opq_t *opq;
 };
 
+static int
+pef_attr_init(ipmi_domain_t *domain, void *cb_data, void **data)
+{
+    locked_list_t *pefl;
+    
+    pefl = locked_list_alloc(ipmi_domain_get_os_hnd(domain));
+    if (!pefl)
+	return ENOMEM;
+
+    *data = pefl;
+    return 0;
+}
+
 static void
 pef_lock(ipmi_pef_t *pef)
 {
@@ -108,6 +132,133 @@ pef_unlock(ipmi_pef_t *pef)
 	pef->os_hnd->unlock(pef->os_hnd, pef->pef_lock);
 }
 
+static void
+pef_get(ipmi_pef_t *pef)
+{
+    pef_lock(pef);
+    pef->refcount++;
+    pef_unlock(pef);
+}
+
+static void internal_destroy_pef(ipmi_pef_t *pef);
+
+static void
+pef_put(ipmi_pef_t *pef)
+{
+    pef_lock(pef);
+    pef->refcount--;
+    if (pef->refcount == 0) {
+	internal_destroy_pef(pef);
+	return;
+    }
+    pef_unlock(pef);
+}
+
+void
+ipmi_pef_ref(ipmi_pef_t *pef)
+{
+    pef_get(pef);
+}
+
+void
+ipmi_pef_deref(ipmi_pef_t *pef)
+{
+    pef_put(pef);
+}
+
+static int
+destroy_pef(void *cb_data, void *item1, void *item2)
+{
+    ipmi_pef_t *pef = item1;
+    pef_lock(pef);
+    pef->in_list = 0;
+    pef_unlock(pef);
+    return LOCKED_LIST_ITER_CONTINUE;
+}
+
+static void
+pef_attr_destroy(void *cb_data, void *data)
+{
+    locked_list_t *pefl = data;
+
+    locked_list_iterate(pefl, destroy_pef, NULL);
+    locked_list_destroy(pefl);
+}
+
+typedef struct iterate_pefs_info_s
+{
+    ipmi_pef_ptr_cb handler;
+    void            *cb_data;
+} iterate_pefs_info_t;
+
+static int
+pefs_handler(void *cb_data, void *item1, void *item2)
+{
+    iterate_pefs_info_t *info = cb_data;
+    info->handler(item1, info->cb_data);
+    pef_put(item1);
+    return LOCKED_LIST_ITER_CONTINUE;
+}
+
+static int
+pefs_prefunc(void *cb_data, void *item1, void *item2)
+{
+    pef_get(item1);
+    return LOCKED_LIST_ITER_CONTINUE;
+}
+
+void
+ipmi_pef_iterate_pefs(ipmi_domain_t       *domain,
+			      ipmi_pef_ptr_cb handler,
+			      void                *cb_data)
+{
+    iterate_pefs_info_t info;
+    ipmi_domain_attr_t  *attr;
+    locked_list_t       *pefs;
+    int                 rv;
+
+    rv = ipmi_domain_find_attribute(domain, IPMI_PEF_ATTR_NAME,
+				    &attr);
+    if (rv)
+	return;
+    pefs = ipmi_domain_attr_get_data(attr);
+
+    info.handler = handler;
+    info.cb_data = cb_data;
+    locked_list_iterate_prefunc(pefs, pefs_prefunc, pefs_handler, &info);
+    ipmi_domain_attr_put(attr);
+}
+
+ipmi_mcid_t
+ipmi_pef_get_mc_id(ipmi_pef_t *pef)
+{
+    return pef->mc;
+}
+
+int
+ipmi_pef_get_name(ipmi_pef_t *pef, char *name, int length)
+{
+    int  slen;
+
+    if (length <= 0)
+	return 0;
+
+    /* Never changes, no lock needed. */
+    slen = strlen(pef->name);
+    if (slen == 0) {
+	if (name)
+	    *name = '\0';
+	goto out;
+    }
+
+    if (name) {
+	memcpy(name, pef->name, slen);
+	name[slen] = '\0';
+    }
+ out:
+    return slen;
+}
+
 static int
 check_pef_response_param(ipmi_pef_t *pef,
 			 ipmi_mc_t  *mc,
@@ -127,7 +278,7 @@ check_pef_response_param(ipmi_pef_t *pef,
 	ipmi_log(IPMI_LOG_ERR_INFO,
 		 "%s: MC went away while PEF op was in progress",
 		 func_name);
-	return ENXIO;
+	return ECANCELED;
     }
 
     if (rsp->data[0] != 0) {
@@ -214,12 +365,24 @@ ipmi_pef_alloc(ipmi_mc_t        *mc,
 	       void             *cb_data,
 	       ipmi_pef_t       **new_pef)
 {
-    ipmi_pef_t    *pef = NULL;
-    int           rv = 0;
-    ipmi_domain_t *domain;
+    ipmi_pef_t         *pef = NULL;
+    int                rv = 0;
+    ipmi_domain_t      *domain = ipmi_mc_get_domain(mc);
+    locked_list_t      *pefl;
+    ipmi_domain_attr_t *attr;
+    int                len, p;
 
     CHECK_MC_LOCK(mc);
 
+    rv = ipmi_domain_register_attribute(domain, IPMI_PEF_ATTR_NAME,
+					pef_attr_init,
+					pef_attr_destroy,
+					NULL,
+					&attr);
+    if (rv)
+	return rv;
+    pefl = ipmi_domain_attr_get_data(attr);
+
     pef = ipmi_mem_alloc(sizeof(*pef));
     if (!pef) {
 	rv = ENOMEM;
@@ -227,8 +390,14 @@ ipmi_pef_alloc(ipmi_mc_t        *mc,
     }
     memset(pef, 0, sizeof(*pef));
 
+    pef->refcount = 1;
+    pef->in_list = 1;
     pef->mc = ipmi_mc_convert_to_id(mc);
-    domain = ipmi_mc_get_domain(mc);
+    pef->domain = ipmi_domain_convert_to_id(domain);
+    len = sizeof(pef->name);
+    p = ipmi_domain_get_name(domain, pef->name, len);
+    len -= p;
+    snprintf(pef->name+p, len, ".%d", ipmi_domain_get_unique_num(domain));
     pef->os_hnd = ipmi_domain_get_os_hnd(domain);
     pef->pef_lock = NULL;
     pef->ready_cb = done;
@@ -246,7 +415,13 @@ ipmi_pef_alloc(ipmi_mc_t        *mc,
 	    goto out;
     }
 
+    if (! locked_list_add(pefl, pef, NULL)) {
+	rv = ENOMEM;
+	goto out;
+    }
+
  out:
+    ipmi_domain_attr_put(attr);
     if (!rv)
 	rv = pef_start_capability_fetch(pef, mc);
 
@@ -270,8 +445,34 @@ internal_destroy_pef(ipmi_pef_t *pef)
 {
     pef->in_destroy = 1;
 
-    /* We don't have to have a valid ipmi to destroy an SEL, the are
-       designed to live after the ipmi has been destroyed. */
+    if (pef->in_list) {
+	ipmi_domain_attr_t *attr;
+	locked_list_t      *pefs;
+	int                rv;
+	rv = ipmi_domain_id_find_attribute(pef->domain,
+					   IPMI_PEF_ATTR_NAME, &attr);
+	if (!rv) {
+	    pef->in_list = 0;
+	    pef->refcount++;
+	    pef_unlock(pef);
+	
+	    pefs = ipmi_domain_attr_get_data(attr);
+
+	    locked_list_remove(pefs, pef, NULL);
+	    ipmi_domain_attr_put(attr);
+	    pef_lock(pef);
+	    /* While we were unlocked, someone may have come in and
+	       grabbed the PEF by iterating the list of PEFs.  That's
+	       ok, we just let them handle the destruction since this
+	       code will not be entered again. */
+	    if (pef->refcount != 1) {
+		pef->refcount--;
+		pef_unlock(pef);
+		return;
+	    }
+	}
+    }
+
     pef_unlock(pef);
 
     if (pef->opq)
@@ -290,29 +491,37 @@ internal_destroy_pef(ipmi_pef_t *pef)
 
 int
 ipmi_pef_destroy(ipmi_pef_t       *pef,
-		 ipmi_pef_done_cb handler,
+		 ipmi_pef_done_cb done,
 		 void             *cb_data)
 {
-    /* We don't need the read lock, because the pef are stand-alone
-       after they are created (except for fetching SELs, of course). */
+    int                rv;
+    ipmi_domain_attr_t *attr;
+    locked_list_t      *pefl;
+
     pef_lock(pef);
+    if (pef->in_list) {
+	pef->in_list = 0;
+	rv = ipmi_domain_id_find_attribute(pef->domain, IPMI_PEF_ATTR_NAME,
+					   &attr);
+	if (!rv) {
+	    pef_unlock(pef);
+	    pefl = ipmi_domain_attr_get_data(attr);
+
+	    locked_list_remove(pefl, pef, NULL);
+	    ipmi_domain_attr_put(attr);
+	    pef_lock(pef);
+	}
+    }
+
     if (pef->destroyed) {
 	pef_unlock(pef);
 	return EINVAL;
     }
     pef->destroyed = 1;
-    pef->destroy_handler = handler;
+    pef_unlock(pef);
+    pef->destroy_handler = done;
     pef->destroy_cb_data = cb_data;
-    if (opq_stuff_in_progress(pef->opq)) {
-	/* It's currently doing something with callbacks, so let it be
-           destroyed in the handler, since we can't cancel the handler
-           or operation. */
-	pef_unlock(pef);
-	return 0;
-    }
-
-    /* This unlocks the lock. */
-    internal_destroy_pef(pef);
+    pef_put(pef);
     return 0;
 }
 
@@ -337,20 +546,22 @@ fetch_complete(ipmi_pef_t *pef, int err, pef_fetch_handler_t *elem)
     if (pef->in_destroy)
 	goto out;
 
+    pef_unlock(pef);
+
     if (elem->handler)
 	elem->handler(pef, err, elem->data, elem->data_len, elem->cb_data);
 
     ipmi_mem_free(elem);
 
-    if (pef->destroyed) {
-	internal_destroy_pef(pef);
-	return;
-    }
+    if (!pef->destroyed)
+	opq_op_done(pef->opq);
 
-    opq_op_done(pef->opq);
+    pef_put(pef);
+    return;
 
  out:
     pef_unlock(pef);
+    pef_put(pef);
 }
 
 
@@ -470,12 +681,14 @@ ipmi_pef_get_parm(ipmi_pef_t      *pef,
     elem->block = block;
     elem->rv = 0;
 
-    pef_lock(pef);
-
-    if (!opq_new_op(pef->opq, start_config_fetch, elem, 0))
+    pef_get(pef);
+    if (!opq_new_op(pef->opq, start_config_fetch, elem, 0)) {
+	pef_put(pef);
 	rv = ENOMEM;
+    }
 
-    pef_unlock(pef);
+    if (rv)
+	ipmi_mem_free(elem);
 
     return rv;
 }
@@ -498,20 +711,27 @@ set_complete(ipmi_pef_t *pef, int err, pef_set_handler_t *elem)
     if (pef->in_destroy)
 	goto out;
 
+    pef_unlock(pef);
+
     if (elem->handler)
 	elem->handler(pef, err, elem->cb_data);
 
     ipmi_mem_free(elem);
 
-    if (pef->destroyed) {
-	internal_destroy_pef(pef);
-	return;
+    pef_lock(pef);
+    if (!pef->destroyed) {
+	pef_unlock(pef);
+	opq_op_done(pef->opq);
+    } else {
+	pef_unlock(pef);
     }
 
-    opq_op_done(pef->opq);
+    pef_put(pef);
+    return;
 
  out:
     pef_unlock(pef);
+    pef_put(pef);
 }
 
 static void
@@ -625,12 +845,15 @@ ipmi_pef_set_parm(ipmi_pef_t       *pef,
     elem->data_len = data_len + 1;
     elem->rv = 0;
 
-    pef_lock(pef);
-
-    if (!opq_new_op(pef->opq, start_config_set, elem, 0))
+    
+    pef_get(pef);
+    if (!opq_new_op(pef->opq, start_config_set, elem, 0)) {
+	pef_put(pef);
 	rv = ENOMEM;
+    }
 
-    pef_unlock(pef);
+    if (rv)
+	ipmi_mem_free(elem);
 
     return rv;
 }
@@ -848,7 +1071,7 @@ static int gctl(ipmi_pef_config_t *pefc, pefparms_t *lp, int err,
 
     pefc->alert_startup_delay_enabled = (data[0] >> 3) & 1;
     pefc->startup_delay_enabled = (data[0] >> 2) & 1;
-    pefc->event_messages_enabled = (data[0] >> 2) & 1;
+    pefc->event_messages_enabled = (data[0] >> 1) & 1;
     pefc->pef_enabled = (data[0] >> 0) & 1;
 
     return 0;
@@ -858,9 +1081,9 @@ static void sctl(ipmi_pef_config_t *pefc, pefparms_t *lp, unsigned char *data,
 		 unsigned int *data_len)
 {
     data[0] = ((pefc->alert_startup_delay_enabled << 3)
-	       || (pefc->startup_delay_enabled << 2)
-	       || (pefc->event_messages_enabled << 1)
-	       || pefc->pef_enabled);
+	       | (pefc->startup_delay_enabled << 2)
+	       | (pefc->event_messages_enabled << 1)
+	       | pefc->pef_enabled);
 }
 
 /* Action Global Control */
@@ -1186,6 +1409,7 @@ static int gnas(ipmi_pef_config_t *pefc, pefparms_t *lp, int err,
     if (num == 0)
 	return 0;
 
+    num++;
     pefc->asks = ipmi_mem_alloc(sizeof(ipmi_ask_t) * num);
     if (!pefc->asks)
 	return ENOMEM;
@@ -1247,6 +1471,7 @@ static int gas(ipmi_pef_config_t *pefc, pefparms_t *lp, int err,
     int           len;
 
     data++; /* Skip the revision byte. */
+    data_len--;
     pos = data[0] & 0x7f;
     t = &(pefc->alert_strings[pos]);
 
@@ -1300,7 +1525,7 @@ static void sas(ipmi_pef_config_t *pefc, pefparms_t *lp, unsigned char *data,
 	return;
     }
 
-    t += (block * 16);
+    t += ((block-1) * 16);
     len = strlen(t);
     if (len >= 16) {
 	memcpy(data+2, t, 16);
@@ -1346,6 +1571,7 @@ err_lock_cleared(ipmi_pef_t *pef,
 
     pefc->done(pef, pefc->err, NULL, pefc->cb_data);
     ipmi_pef_free_config(pefc);
+    pef_put(pef);
 }
 
 static void
@@ -1478,6 +1704,7 @@ got_parm(ipmi_pef_t     *pef,
 	    /* End of string, either a subsize-block or a nil
 	       character in the data. */
 	    pefc->curr_sel++;
+	    pefc->curr_block = 1;
 	    if (pefc->curr_sel >= pefc->num_alert_strings)
 		goto done;
 	} else {
@@ -1495,7 +1722,7 @@ got_parm(ipmi_pef_t     *pef,
 	goto next_parm;
 
     err = ipmi_pef_get_parm(pef, pefc->curr_parm, pefc->curr_sel, 
-		    pefc->curr_block, got_parm, pefc);
+			    pefc->curr_block, got_parm, pefc);
     if (err)
 	goto done;
 
@@ -1513,15 +1740,17 @@ got_parm(ipmi_pef_t     *pef,
 	data[0] = 0;
 	err = ipmi_pef_set_parm(pef, 0, data, 1, err_lock_cleared, pefc);
 	if (err) {
-	    ipmi_pef_free_config(pefc);
 	    ipmi_log(IPMI_LOG_ERR_INFO,
 		     "pef.c(got_parm): Error trying to clear lock: %x",
 		     err);
 	    pefc->done(pef, pefc->err, NULL, pefc->cb_data);
 	    ipmi_pef_free_config(pefc);
+	    pef_put(pef);
 	}
-    } else
+    } else {
 	pefc->done(pef, 0, pefc, pefc->cb_data);
+	pef_put(pef);
+    }
 }
 
 static void 
@@ -1539,6 +1768,7 @@ lock_done(ipmi_pef_t *pef,
 	/* Someone else has the lock, return EAGAIN. */
 	pefc->done(pef, err, NULL, pefc->cb_data);
 	ipmi_pef_free_config(pefc);
+	pef_put(pef);
 	return;
     } else if (err) {
 	ipmi_log(IPMI_LOG_ERR_INFO,
@@ -1547,6 +1777,7 @@ lock_done(ipmi_pef_t *pef,
 		 err);
 	pefc->done(pef, err, NULL, pefc->cb_data);
 	ipmi_pef_free_config(pefc);
+	pef_put(pef);
 	return;
     }
 
@@ -1565,12 +1796,12 @@ lock_done(ipmi_pef_t *pef,
 	data[0] = 0;
 	rv = ipmi_pef_set_parm(pef, 0, data, 1, err_lock_cleared, pefc);
 	if (rv) {
-	    ipmi_pef_free_config(pefc);
 	    ipmi_log(IPMI_LOG_ERR_INFO,
 		     "pef.c(lock_done): Error trying to clear lock: %x",
 		     err);
 	    pefc->done(pef, pefc->err, NULL, pefc->cb_data);
 	    ipmi_pef_free_config(pefc);
+	    pef_put(pef);
 	}
     }
 }
@@ -1593,13 +1824,16 @@ int ipmi_pef_get_config(ipmi_pef_t             *pef,
     pefc->done = done;
     pefc->cb_data = cb_data;
     pefc->my_pef = pef;
-    pefc->lock_supported = 1; /* Assume it works */
+    pefc->lock_supported = 1; /* Assume it works. */
 
     /* First grab the lock */
     data[0] = 1; /* Set in progress. */
+    pef_get(pef);
     rv = ipmi_pef_set_parm(pef, 0, data, 1, lock_done, pefc);
-    if (rv)
+    if (rv) {
 	ipmi_pef_free_config(pefc);
+	pef_put(pef);
+    }
 
     return rv;
 }
@@ -1615,6 +1849,7 @@ set_clear(ipmi_pef_t *pef,
 	err = pefc->err;
     pefc->set_done(pef, err, pefc->cb_data);
     ipmi_pef_free_config(pefc);
+    pef_put(pef);
 }
 
 static void 
@@ -1674,9 +1909,9 @@ set_done(ipmi_pef_t *pef,
 
     case IPMI_PEFPARM_EVENT_FILTER_TABLE:
 	pefc->curr_sel++;
-	if (pefc->curr_sel >= pefc->num_event_filters) {
+	if (pefc->curr_sel > pefc->num_event_filters) {
 	    pefc->curr_parm++;
-	    pefc->curr_sel = 0;
+	    pefc->curr_sel = 1;
 	}
 	data[0] = pefc->curr_sel;
 	break;
@@ -1693,7 +1928,7 @@ set_done(ipmi_pef_t *pef,
 
     case IPMI_PEFPARM_ALERT_POLICY_TABLE:
 	pefc->curr_sel++;
-	if (pefc->curr_sel >= pefc->num_alert_policies) {
+	if (pefc->curr_sel > pefc->num_alert_policies) {
 	    pefc->curr_parm++;
 	    pefc->curr_sel = 0;
 	}
@@ -1714,13 +1949,13 @@ set_done(ipmi_pef_t *pef,
 	    pefc->curr_parm++;
 	    pefc->curr_sel = 0;
 	    pefc->curr_block = 1;
+	    data[1] = pefc->curr_block;
 	}
 	data[0] = pefc->curr_sel;
-	data[1] = pefc->curr_block;
 	break;
 
     case IPMI_PEFPARM_ALERT_STRING:
-	/* curr_sel increment is done write after the send formatting,
+	/* curr_sel increment is done right after the send formatting,
 	   because we don't know until there if it's the end of the
 	   string. */
 	if (pefc->curr_sel >= pefc->num_alert_strings)
@@ -1757,7 +1992,7 @@ set_done(ipmi_pef_t *pef,
 	    /* End of string, either a subsize-block or a nil
 	       character in the data. */
 	    pefc->curr_sel++;
-	    pefc->curr_block = 0;
+	    pefc->curr_block = 1;
 	} else {
 	    pefc->curr_block++;
 	}
@@ -1881,7 +2116,10 @@ ipmi_pef_set_config(ipmi_pef_t        *pef,
     lp = &(pefparms[pefc->curr_parm]);
     length = lp->length;
     lp->set_handler(pefc, lp, data, &length);
+    pef_get(pef);
     rv = ipmi_pef_set_parm(pef, pefc->curr_parm, data, length, set_done, pefc);
+    if (rv)
+	pef_put(pef);
  out:
     if (rv) {
 	ipmi_pef_free_config(pefc);
@@ -1910,6 +2148,7 @@ lock_cleared(ipmi_pef_t *pef,
 	cl->done(pef, err, cl->cb_data);
 
     ipmi_mem_free(cl);
+    pef_put(pef);
 }
 
 int
@@ -1937,8 +2176,10 @@ ipmi_pef_clear_lock(ipmi_pef_t        *pef,
     cl->cb_data = cb_data;
 
     data[0] = 0; /* Clear the lock. */
+    pef_get(pef);
     rv = ipmi_pef_set_parm(pef, 0, data, 1, lock_cleared, cl);
     if (rv) {
+	pef_put(pef);
 	ipmi_mem_free(cl);
     } else if (pefc) {
 	pefc->pef_locked = 0;
@@ -1997,7 +2238,7 @@ int \
 ipmi_pefconfig_get_ ## n(ipmi_pef_config_t *pefc, unsigned int *val) \
 { \
     if (!pefc->n ## _supported) \
-	return ENOTSUP; \
+	return ENOSYS; \
     *val = pefc->n; \
     return 0; \
 } \
@@ -2005,7 +2246,7 @@ int \
 ipmi_pefconfig_set_ ## n(ipmi_pef_config_t *pefc, unsigned int val) \
 { \
     if (!pefc->n ## _supported) \
-	return ENOTSUP; \
+	return ENOSYS; \
     pefc->n = val; \
     return 0; \
 }
@@ -2013,6 +2254,33 @@ ipmi_pefconfig_set_ ## n(ipmi_pef_config_t *pefc, unsigned int val) \
 PEF_BYTE_SUPPORT(startup_delay);
 PEF_BYTE_SUPPORT(alert_startup_delay);
 
+PEF_BIT(guid_enabled);
+
+int
+ipmi_pefconfig_get_guid_val(ipmi_pef_config_t *pefc,
+			    unsigned char     *data,
+			    unsigned int      *data_len)
+{
+    if (*data_len < 16) {
+	*data_len = 16;
+        return EBADF;
+    }
+    memcpy(data, pefc->guid, 16);
+    *data_len = 16;
+    return 0;
+}
+
+int
+ipmi_pefconfig_set_guid_val(ipmi_pef_config_t *pefc,
+			    unsigned char     *data,
+			    unsigned int      data_len)
+{
+    if (data_len != 16)
+	return EINVAL;
+    memcpy(pefc->guid, data, 16);
+    return 0;
+}
+
 int
 ipmi_pefconfig_get_guid(ipmi_pef_config_t *pefc,
 			unsigned int      *enabled,
@@ -2133,18 +2401,24 @@ PEF_SUB_BYTE(asks, num_alert_strings, alert_string_set);
 
 int
 ipmi_pefconfig_get_alert_string(ipmi_pef_config_t *pefc, unsigned int sel,
-				unsigned char *val, unsigned int len)
+				unsigned char *val, unsigned int *len)
 {
+    unsigned int olen = *len;
+    unsigned int rlen;
+
     if (sel >= pefc->num_alert_strings)
 	return EINVAL;
-    if (len == 0)
-	return EINVAL;
     if (! pefc->alert_strings[sel]) {
+	*len = 1;
+	if (olen == 0)
+	    return EBADF;
 	*val = '\0';
 	return 0;
     }
-    if (strlen((char *) pefc->alert_strings[sel])+1 < len)
-	return EAGAIN;
+    rlen = strlen((char *) pefc->alert_strings[sel]) + 1;
+    *len = rlen;
+    if (rlen > olen)
+	return EBADF;
     strcpy(val, pefc->alert_strings[sel]);
     return 0;
 }
@@ -2167,3 +2441,345 @@ ipmi_pefconfig_set_alert_string(ipmi_pef_config_t *pefc, unsigned int sel,
 	ipmi_mem_free(s);
     return 0;
 }
+
+
+typedef struct pefparm_gendata_s
+{
+    enum ipmi_pefconf_val_type_e datatype;
+    unsigned char *fname;
+
+    union {
+	struct {
+	    unsigned int (*gval)(ipmi_pef_config_t *pefc);
+	    int (*gval_v)(ipmi_pef_config_t *pefc, unsigned int *val);
+	    int (*gval_iv)(ipmi_pef_config_t *pefc, unsigned int idx,
+			   unsigned int *val);
+	    int (*sval)(ipmi_pef_config_t *pefc, unsigned int val);
+	    int (*sval_v)(ipmi_pef_config_t *pefc, unsigned int val);
+	    int (*sval_iv)(ipmi_pef_config_t *pefc, unsigned int idx,
+			   unsigned int val);
+	} ival;
+	struct {
+	    int (*gval_v)(ipmi_pef_config_t *pefc, unsigned char *data,
+			  unsigned int *data_len);
+	    int (*gval_iv)(ipmi_pef_config_t *pefc, unsigned int idx,
+			   unsigned char *data, unsigned int *data_len);
+	    int (*sval_v)(ipmi_pef_config_t *pefc, unsigned char *data,
+			  unsigned int data_len);
+	    int (*sval_iv)(ipmi_pef_config_t *pefc, unsigned int idx,
+			   unsigned char *data, unsigned int data_len);
+	} dval;
+	struct {
+	    int (*gval_v)(ipmi_pef_config_t *pefc, unsigned char *data,
+			  unsigned int *data_len);
+	    int (*gval_iv)(ipmi_pef_config_t *pefc, unsigned int idx,
+			   unsigned char *data, unsigned int *data_len);
+	    int (*sval_v)(ipmi_pef_config_t *pefc, unsigned char *data);
+	    int (*sval_iv)(ipmi_pef_config_t *pefc, unsigned int idx,
+			   unsigned char *data);
+	} sval;
+    } u;
+    unsigned int (*iv_cnt)(ipmi_pef_config_t *pefc);
+} pefparm_gendata_t;
+
+#define F_BOOLR(name) \
+	{ .datatype = IPMI_PEFCONFIG_BOOL, .fname = #name, \
+	  .u = { .ival = { .gval = ipmi_pefconfig_get_ ## name }}}
+#define F_BOOL(name) \
+	{ .datatype = IPMI_PEFCONFIG_BOOL, .fname = #name, \
+	  .u = { .ival = { .gval = ipmi_pefconfig_get_ ## name, \
+			   .sval = ipmi_pefconfig_set_ ## name }}}
+#define F_INTR(name) \
+	{ .datatype = IPMI_PEFCONFIG_INT, .fname = #name, \
+	  .u = { .ival = { .gval = ipmi_pefconfig_get_ ## name }}}
+#define F_INT(name) \
+	{ .datatype = IPMI_PEFCONFIG_INT, .fname = #name, \
+	  .u = { .ival = { .gval = ipmi_pefconfig_get_ ## name, \
+			   .sval = ipmi_pefconfig_set_ ## name }}}
+#define F_INTV(name) \
+	{ .datatype = IPMI_PEFCONFIG_INT, .fname = #name, \
+	  .u = { .ival = { .gval_v = ipmi_pefconfig_get_ ## name, \
+			   .sval_v = ipmi_pefconfig_set_ ## name }}}
+#define F_INTIV(name, gcnt) \
+	{ .datatype = IPMI_PEFCONFIG_INT, .fname = #name, \
+	  .u = { .ival = { .gval_iv = ipmi_pefconfig_get_ ## name, \
+			   .sval_iv = ipmi_pefconfig_set_ ## name }}, \
+	  .iv_cnt = gcnt }
+#define F_BOOLV(name) \
+	{ .datatype = IPMI_PEFCONFIG_BOOL, .fname = #name, \
+	  .u = { .ival = { .gval_v = ipmi_pefconfig_get_ ## name, \
+			   .sval_v = ipmi_pefconfig_set_ ## name }}}
+#define F_BOOLIV(name, gcnt) \
+	{ .datatype = IPMI_PEFCONFIG_BOOL, .fname = #name, \
+	  .u = { .ival = { .gval_iv = ipmi_pefconfig_get_ ## name, \
+			   .sval_iv = ipmi_pefconfig_set_ ## name }}, \
+	  .iv_cnt = gcnt }
+#define F_DATA(name) \
+	{ .datatype = IPMI_PEFCONFIG_DATA, .fname = #name, \
+	  .u = { .dval = { .gval_v = ipmi_pefconfig_get_ ## name, \
+			   .sval_v = ipmi_pefconfig_set_ ## name }}}
+#define F_STRIV(name, gcnt) \
+	{ .datatype = IPMI_PEFCONFIG_STR, .fname = #name, \
+	  .u = { .sval = { .gval_iv = ipmi_pefconfig_get_ ## name, \
+			   .sval_iv = ipmi_pefconfig_set_ ## name }}, \
+	  .iv_cnt = gcnt }
+
+static pefparm_gendata_t gdata[] =
+{
+    F_BOOL(alert_startup_delay_enabled),			/* 0 */
+    F_BOOL(startup_delay_enabled),
+    F_BOOL(event_messages_enabled),
+    F_BOOL(pef_enabled),
+    F_BOOL(diagnostic_interrupt_enabled),
+    F_BOOL(oem_action_enabled),					/* 5 */
+    F_BOOL(power_cycle_enabled),
+    F_BOOL(reset_enabled),
+    F_BOOL(power_down_enabled),
+    F_BOOL(alert_enabled),
+    F_INTV(startup_delay),					/* 10 */
+    F_INTV(alert_startup_delay),
+    F_BOOL(guid_enabled),
+    F_DATA(guid_val),
+    F_INTR(num_event_filters),
+    F_BOOLIV(enable_filter, ipmi_pefconfig_get_num_event_filters), /* 15 */
+    F_INTIV(filter_type, ipmi_pefconfig_get_num_event_filters),
+    F_BOOLIV(diagnostic_interrupt, ipmi_pefconfig_get_num_event_filters),
+    F_BOOLIV(oem_action, ipmi_pefconfig_get_num_event_filters),
+    F_BOOLIV(power_cycle, ipmi_pefconfig_get_num_event_filters),
+    F_BOOLIV(reset, ipmi_pefconfig_get_num_event_filters),	  /* 20 */
+    F_BOOLIV(power_down, ipmi_pefconfig_get_num_event_filters),
+    F_BOOLIV(alert, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(alert_policy_number, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(event_severity, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(generator_id_addr, ipmi_pefconfig_get_num_event_filters), /* 25 */
+    F_INTIV(generator_id_channel_lun, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(sensor_type, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(sensor_number, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(event_trigger, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data1_offset_mask, ipmi_pefconfig_get_num_event_filters), /* 30 */
+    F_INTIV(data1_mask, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data1_compare1, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data1_compare2, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data2_mask, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data2_compare1, ipmi_pefconfig_get_num_event_filters), /* 35 */
+    F_INTIV(data2_compare2, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data3_mask, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data3_compare1, ipmi_pefconfig_get_num_event_filters),
+    F_INTIV(data3_compare2, ipmi_pefconfig_get_num_event_filters),
+    F_INTR(num_alert_policies),					   /* 40 */
+    F_INTIV(policy_num, ipmi_pefconfig_get_num_alert_policies),
+    F_BOOLIV(enabled, ipmi_pefconfig_get_num_alert_policies),
+    F_INTIV(policy, ipmi_pefconfig_get_num_alert_policies),
+    F_INTIV(channel, ipmi_pefconfig_get_num_alert_policies),
+    F_INTIV(destination_selector, ipmi_pefconfig_get_num_alert_policies), /* 45 */
+    F_BOOLIV(alert_string_event_specific, ipmi_pefconfig_get_num_alert_policies),
+    F_INTIV(alert_string_selector, ipmi_pefconfig_get_num_alert_policies),
+    F_INTR(num_alert_strings),
+    F_INTIV(event_filter, ipmi_pefconfig_get_num_alert_strings),
+    F_INTIV(alert_string_set, ipmi_pefconfig_get_num_alert_strings), /* 50 */
+    F_STRIV(alert_string, ipmi_pefconfig_get_num_alert_strings),
+};
+#define NUM_GDATA_ENTRIES (sizeof(gdata) / sizeof(pefparm_gendata_t))
+
+int
+ipmi_pefconfig_get_val(ipmi_pef_config_t *pefc,
+		       unsigned int      parm,
+		       const char        **name,
+		       int               *index,
+		       enum ipmi_pefconf_val_type_e *valtype,
+		       unsigned int      *ival,
+		       unsigned char     **dval,
+		       unsigned int      *dval_len)
+{
+    unsigned int  curr = *index;
+    unsigned int  count;
+    int           rv = 0;
+    unsigned char *data;
+    unsigned int  data_len;
+
+    if (parm >= NUM_GDATA_ENTRIES)
+	return EINVAL;
+    if (valtype)
+	*valtype = gdata[parm].datatype;
+    if (name)
+	*name = gdata[parm].fname;
+
+    if (gdata[parm].iv_cnt) {
+	count = gdata[parm].iv_cnt(pefc);
+	if (curr >= count) {
+	    *index = -1;
+	    return E2BIG;
+	}
+
+	if (curr+1 == count)
+	    *index = -1;
+	else
+	    *index = curr+1;
+    }
+
+    switch (gdata[parm].datatype) {
+    case IPMI_PEFCONFIG_INT:
+    case IPMI_PEFCONFIG_BOOL:
+	if (!ival)
+	    break;
+	if (gdata[parm].u.ival.gval)
+	    *ival = gdata[parm].u.ival.gval(pefc);
+	else if (gdata[parm].u.ival.gval_v)
+	    rv = gdata[parm].u.ival.gval_v(pefc, ival);
+	else if (gdata[parm].u.ival.gval_iv)
+	    rv = gdata[parm].u.ival.gval_iv(pefc, curr, ival);
+	else
+	    rv = ENOSYS;
+	break;
+
+    case IPMI_PEFCONFIG_DATA:
+	data_len = 0;
+	if (gdata[parm].u.dval.gval_v)
+	    rv = gdata[parm].u.dval.gval_v(pefc, NULL, &data_len);
+	else if (gdata[parm].u.dval.gval_iv)
+	    rv = gdata[parm].u.dval.gval_iv(pefc, curr, NULL, &data_len);
+	else
+	    rv = ENOSYS;
+	if (rv && (rv != EBADF))
+	    break;
+	if (data_len == 0)
+	    data = ipmi_mem_alloc(1);
+	else
+	    data = ipmi_mem_alloc(data_len);
+	if (gdata[parm].u.dval.gval_v)
+	    rv = gdata[parm].u.dval.gval_v(pefc, data, &data_len);
+	else if (gdata[parm].u.dval.gval_iv)
+	    rv = gdata[parm].u.dval.gval_iv(pefc, curr, data, &data_len);
+	if (rv) {
+	    ipmi_mem_free(data);
+	    break;
+	}
+	if (dval)
+	    *dval = data;
+	if (dval_len)
+	    *dval_len = data_len;
+	break;
+
+    case IPMI_PEFCONFIG_STR:
+	data_len = 0;
+	if (gdata[parm].u.sval.gval_v)
+	    rv = gdata[parm].u.sval.gval_v(pefc, NULL, &data_len);
+	else if (gdata[parm].u.sval.gval_iv)
+	    rv = gdata[parm].u.sval.gval_iv(pefc, curr, NULL, &data_len);
+	else
+	    rv = ENOSYS;
+	if (rv && (rv != EBADF))
+	    break;
+	if (data_len == 0)
+	    data = ipmi_mem_alloc(1);
+	else
+	    data = ipmi_mem_alloc(data_len);
+	if (gdata[parm].u.sval.gval_v)
+	    rv = gdata[parm].u.sval.gval_v(pefc, data, &data_len);
+	else if (gdata[parm].u.sval.gval_iv)
+	    rv = gdata[parm].u.sval.gval_iv(pefc, curr, data, &data_len);
+	if (rv) {
+	    ipmi_mem_free(data);
+	    break;
+	}
+	if (dval)
+	    *dval = data;
+	if (dval_len)
+	    *dval_len = data_len;
+    }
+
+    return rv;
+}
+
+int
+ipmi_pefconfig_set_val(ipmi_pef_config_t *pefc,
+		       unsigned int      parm,
+		       int               index,
+		       unsigned int      ival,
+		       unsigned char     *dval,
+		       unsigned int      dval_len)
+{
+    unsigned int  count;
+    int           rv = 0;
+
+    if (parm >= NUM_GDATA_ENTRIES)
+	return EINVAL;
+
+    if (gdata[parm].iv_cnt) {
+	count = gdata[parm].iv_cnt(pefc);
+	if (index >= count)
+	    return E2BIG;
+    }
+
+    switch (gdata[parm].datatype) {
+    case IPMI_PEFCONFIG_INT:
+    case IPMI_PEFCONFIG_BOOL:
+	if (!ival)
+	    break;
+	if (gdata[parm].u.ival.sval)
+	    rv = gdata[parm].u.ival.sval(pefc, ival);
+	else if (gdata[parm].u.ival.sval_v)
+	    rv = gdata[parm].u.ival.sval_v(pefc, ival);
+	else if (gdata[parm].u.ival.sval_iv)
+	    rv = gdata[parm].u.ival.sval_iv(pefc, index, ival);
+	else
+	    rv = ENOSYS;
+	break;
+
+    case IPMI_PEFCONFIG_DATA:
+	if (gdata[parm].u.dval.sval_v)
+	    rv = gdata[parm].u.dval.sval_v(pefc, dval, dval_len);
+	else if (gdata[parm].u.dval.sval_iv)
+	    rv = gdata[parm].u.dval.sval_iv(pefc, index, dval, dval_len);
+	else
+	    rv = ENOSYS;
+	break;
+
+    case IPMI_PEFCONFIG_STR:
+	if (gdata[parm].u.sval.sval_v)
+	    rv = gdata[parm].u.sval.sval_v(pefc, dval);
+	else if (gdata[parm].u.sval.sval_iv)
+	    rv = gdata[parm].u.sval.sval_iv(pefc, index, dval);
+	else
+	    rv = ENOSYS;
+	break;
+    }
+
+    return rv;
+}
+
+
+void
+ipmi_pefconfig_data_free(void *data)
+{
+    ipmi_mem_free(data);
+}
+
+unsigned int
+ipmi_pefconfig_str_to_parm(char *name)
+{
+    int i;
+    for (i=0; i<NUM_GDATA_ENTRIES; i++) {
+	if (strcmp(name, gdata[i].fname) == 0)
+	    return i;
+    }
+    return -1;
+}
+
+const char *
+ipmi_pefconfig_parm_to_str(unsigned int parm)
+{
+    if (parm >= NUM_GDATA_ENTRIES)
+	return NULL;
+    return gdata[parm].fname;
+}
+
+int
+ipmi_pefconfig_parm_to_type(unsigned int                 parm,
+			    enum ipmi_pefconf_val_type_e *valtype)
+{
+    if (parm >= NUM_GDATA_ENTRIES)
+	return EINVAL;
+    *valtype = gdata[parm].datatype;
+    return 0;
+}
