@@ -839,7 +839,7 @@ int main(int argc, char *argv[])
 }
 
 void
-posix_vlog(char *format, enum ipmi_log_type_e log_type, va_list ap)
+posix_vlog(const char *format, enum ipmi_log_type_e log_type, va_list ap)
 {
     int do_nl = 1;
 
