@@ -47,9 +47,17 @@
 
 #include <OpenIPMI/ipmi_posix.h>
 
+/* CHEAP HACK - we don't want the user to have to provide this any
+   more. */
+extern void posix_vlog(const char           *format,
+		       enum ipmi_log_type_e log_type,
+		       va_list              ap);
+#pragma weak posix_vlog
+
 typedef struct iposix_info_s
 {
     selector_t *sel;
+    os_vlog_t  log_handler;
 #ifdef HAVE_GDBM
     char *gdbm_filename;
     GDBM_FILE gdbmf;
@@ -258,25 +266,82 @@ get_random(os_handler_t *handler, void *data, unsigned int len)
 }
 
 static void
-sposix_log(os_handler_t         *handler,
-	   enum ipmi_log_type_e log_type,
-	   char                 *format,
-	   ...)
+default_vlog(const char           *format,
+	     enum ipmi_log_type_e log_type,
+	     va_list              ap)
 {
-    va_list ap;
+    int do_nl = 1;
 
-    va_start(ap, format);
-    posix_vlog(format, log_type, ap);
-    va_end(ap);
+    switch(log_type)
+    {
+	case IPMI_LOG_INFO:
+	    fprintf(stderr, "INFO: ");
+	    break;
+
+	case IPMI_LOG_WARNING:
+	    fprintf(stderr, "WARN: ");
+	    break;
+
+	case IPMI_LOG_SEVERE:
+	    fprintf(stderr, "SEVR: ");
+	    break;
+
+	case IPMI_LOG_FATAL:
+	    fprintf(stderr, "FATL: ");
+	    break;
+
+	case IPMI_LOG_ERR_INFO:
+	    fprintf(stderr, "EINF: ");
+	    break;
+
+	case IPMI_LOG_DEBUG_START:
+	    do_nl = 0;
+	    /* FALLTHROUGH */
+	case IPMI_LOG_DEBUG:
+	    fprintf(stderr, "DEBG: ");
+	    break;
+
+	case IPMI_LOG_DEBUG_CONT:
+	    do_nl = 0;
+	    /* FALLTHROUGH */
+	case IPMI_LOG_DEBUG_END:
+	    break;
+    }
+
+    vfprintf(stderr, format, ap);
+
+    if (do_nl)
+	fprintf(stderr, "\n");
 }
 
 static void
 sposix_vlog(os_handler_t         *handler,
 	    enum ipmi_log_type_e log_type,
-	    char                 *format,
+	    const char           *format,
 	    va_list              ap)
 {
-    posix_vlog(format, log_type, ap);
+    iposix_info_t *info = handler->internal_data;
+    os_vlog_t     log_handler = info->log_handler;
+
+    if (log_handler)
+	log_handler(handler, format, log_type, ap);
+    else if (posix_vlog)
+	posix_vlog(format, log_type, ap);
+    else
+	default_vlog(format, log_type, ap);
+}
+
+static void
+sposix_log(os_handler_t         *handler,
+	   enum ipmi_log_type_e log_type,
+	   const char           *format,
+	   ...)
+{
+    va_list ap;
+
+    va_start(ap, format);
+    sposix_vlog(handler, log_type, format, ap);
+    va_end(ap);
 }
 
 static int
@@ -426,6 +491,14 @@ set_gdbm_filename(os_handler_t *os_hnd, char *name)
 }
 #endif
 
+void sset_log_handler(os_handler_t *handler,
+		      os_vlog_t    log_handler)
+{
+    iposix_info_t *info = handler->internal_data;
+
+    info->log_handler = log_handler;
+}
+
 static os_handler_t ipmi_posix_os_handler =
 {
     .mem_alloc = posix_malloc,
@@ -448,6 +521,7 @@ static os_handler_t ipmi_posix_os_handler =
     .database_free = database_free,
     .database_set_filename = set_gdbm_filename,
 #endif
+    .set_log_handler = sset_log_handler,
 };
 
 os_handler_t *
